@@ -1,4 +1,4 @@
-__version__ = '0.0.31'
+__version__ = '0.0.32'
 
 DB_VERSION = 1
 
@@ -6,6 +6,8 @@ DEFAULT_FMT = 'json'
 
 DEFAULT_HTTP_TIMEOUT = 5
 
+DEFAULT_CACHE_SIZE = 1000
+
 FMTS = ['json', 'yaml', 'msgpack', 'cbor', 'pickle']
 
 import threading
@@ -16,6 +18,7 @@ RLock = threading.RLock
 
 from pathlib import Path
 from functools import partial
+from cachetools import LRUCache
 
 import os
 
@@ -154,9 +157,11 @@ class YEDB():
             http_username: http username
             http_password: http password
             http_auth: auth type (basic or digest)
+            cache_size: item cache size
         """
         path = str(dbpath)
         self.auto_repair = kwargs.get('auto_repair')
+        self.cache = LRUCache(kwargs.get('cache_size', DEFAULT_CACHE_SIZE))
         if debug:
             logger.debug('initializing db')
             logger.debug(f'path: {path}')
@@ -349,23 +354,43 @@ class YEDB():
         if isinstance(s, str) and not s.endswith('\n'):
             s += '\n'
         if self.checksums:
-            val = self.calc_digest(s)
+            checksum = self.calc_digest(s)
+            val = checksum
             if stime is None:
                 stime = time_ns()
             if self.checksum_binary:
                 val += stime.to_bytes(8, 'little') + s
             else:
                 val += '\n' + stime.to_bytes(8, 'little').hex() + '\n' + s
-            return val
+            return val, checksum, stime
         else:
-            return s
+            return s, None, None
+
+    def _purge_cache_by_path(self, path):
+        if path != '' and not path.endswith('/'):
+            path += '/'
+        to_purge = set()
+        with self.lock:
+            for k in self.cache:
+                if k.startswith(path) and k in self.cache:
+                    to_purge.add(k)
+            for k in to_purge:
+                try:
+                    del self.cache[k]
+                except KeyError:
+                    pass
+                try:
+                    del self._key_locks[k]
+                except KeyError:
+                    pass
 
     def info(self):
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             d = self.dbinfo.copy()
             d['repair_recommended'] = self.repair_recommended
+            d['cached'] = len(self.cache)
             try:
                 d.update(self.meta_info.copy())
             except:
@@ -415,7 +440,7 @@ class YEDB():
                             checksums is None else False
                     new_db._init_meta()
                     new_db.meta_info['created'] = self.meta_info['created']
-                    for key in self.list_subkeys():
+                    for key in self.list_subkeys(hidden=True):
                         val = self._get(key, _extended_info=True)
                         new_db.set(key, val[0], stime=val[3])
                         yield (key, True)
@@ -472,6 +497,7 @@ class YEDB():
         if debug:
             logger.debug(f'opening database {self.db}')
         with self.lock:
+            self.cache.clear()
             self._parse_options(kwargs)
             if _skip_meta:
                 self.checksums = self.default_checksums
@@ -551,6 +577,7 @@ class YEDB():
                 except FileNotFoundError:
                     pass
             self._opened = False
+            self.cache.clear()
             if self.auto_flush:
                 self._sync_dirs([self.db])
 
@@ -575,12 +602,12 @@ class YEDB():
             key: key name
             value: key value
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         name = self._fmt_key(key)
         if debug:
             logger.debug(f'setting key {name}={_format_debug_value(value)}')
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             if not name:
                 raise ValueError('key name not specified')
             try:
@@ -603,9 +630,9 @@ class YEDB():
                             return
                     except KeyError:
                         pass
-                self._write(key_file,
-                            self._dump_value(value, stime=stime),
-                            flush=flush or self.auto_flush)
+                val, checksum, stime = self._dump_value(value, stime=stime)
+                self._write(key_file, val, flush=flush or self.auto_flush)
+                self.cache[key] = (value, checksum, stime)
 
     def copy(self, key, dst_key, delete=False):
         """
@@ -623,11 +650,11 @@ class YEDB():
         """
         Rename key or category to new
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         if debug:
             logger.debug(f'renaming key {key} to {dst_key}')
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             name = self._fmt_key(key)
             if not name:
                 raise ValueError('key name not specified')
@@ -663,6 +690,8 @@ class YEDB():
                     try:
                         key_file.rename(dst_key_file)
                         renamed = True
+                        if key in self.cache:
+                            self.cache[dst_key] = self.cache.pop(key)
                         if flush or self.auto_flush:
                             self._sync_dirs(
                                 {key_file.parent, dst_key_file.parent})
@@ -674,6 +703,7 @@ class YEDB():
                     dst_d = dst_key_file.with_suffix('')
                     try:
                         d.rename(dst_d)
+                        self._purge_cache_by_path(key.rsplit('/', 1)[0])
                         if flush or self.auto_flush:
                             self._sync_dirs({d.parent, dst_d.parent})
                     except FileNotFoundError:
@@ -735,12 +765,12 @@ class YEDB():
              default=KeyError,
              _extended_info=False,
              _check_exists_only=False):
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         name = self._fmt_key(name)
         if debug:
             logger.debug(f'reading key {name} value')
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             try:
                 l = self._key_locks[name]
             except KeyError:
@@ -757,20 +787,27 @@ class YEDB():
                     try:
                         if debug:
                             logger.debug(f'reading key {name} file {key_file}')
-                        s = self.read(key_file)
-                        if self.checksums and _extended_info:
-                            if self.checksum_binary:
-                                checksum = s[:32]
-                                stime = s[32:40]
-                            else:
-                                checksum, stime, _ = s.split(maxsplit=2)
-                                checksum = bytes.fromhex(checksum)
-                                stime = bytes.fromhex(stime)
-                            stime = int.from_bytes(stime, 'little')
+                        if name in self.cache:
+                            if debug:
+                                logger.debug(f'found cached key {name}')
+                            value, checksum, stime = self.cache[name]
                         else:
-                            checksum = None
-                            stime = None
-                        return (self._load_value(s),
+                            s = self.read(key_file)
+                            if self.checksums and _extended_info:
+                                if self.checksum_binary:
+                                    checksum = s[:32]
+                                    stime = s[32:40]
+                                else:
+                                    checksum, stime, _ = s.split(maxsplit=2)
+                                    checksum = bytes.fromhex(checksum)
+                                    stime = bytes.fromhex(stime)
+                                stime = int.from_bytes(stime, 'little')
+                            else:
+                                checksum = None
+                                stime = None
+                            value = self._load_value(s)
+                            self.cache[name] = (value, checksum, stime)
+                        return (value,
                                 key_file.stat() if _extended_info else None,
                                 checksum, stime,
                                 key_file if _extended_info else None)
@@ -787,9 +824,9 @@ class YEDB():
         Args:
             key: key name
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             name = self._fmt_key(key)
             if not name:
                 raise ValueError('key name not specified')
@@ -811,9 +848,9 @@ class YEDB():
         Args:
             key: key name
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             name = self._fmt_key(key)
             if not name:
                 raise ValueError('key name not specified')
@@ -841,8 +878,6 @@ class YEDB():
             key: key name
             recursive: also delete subkeys
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         name = self._fmt_key(key)
         if name == '' and not recursive:
             return
@@ -850,6 +885,8 @@ class YEDB():
             logger.debug(f'deleting key {name}')
         dts = set()
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             try:
                 l = self._key_locks[name]
             except KeyError:
@@ -881,11 +918,17 @@ class YEDB():
                     del self._key_locks[name]
                 except KeyError:
                     pass
+                try:
+                    del self.cache[name]
+                except KeyError:
+                    pass
                 for p in [keydir] + list(keydir.parents):
                     if p == self.db:
                         break
                     try:
                         p.rmdir()
+                        self._purge_cache_by_path(
+                            p.absolute().as_posix()[self._dbpath_len:])
                         if flush or self.auto_flush:
                             dts.add(p.parent)
                     except OSError:
@@ -945,8 +988,6 @@ class YEDB():
             Generator object with tuples (key, True|False) where True means a
             key is repaired and False means a key is purged.
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         if debug:
             logger.debug(f'repair operation requested')
         if not self.meta_info['checksums']:
@@ -955,6 +996,9 @@ class YEDB():
         # repair
         dts = set()
         with self.lock:
+            self.cache.clear()
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             # find possible valid keys
             for d in self.db.glob('**/*.tmp'):
                 try:
@@ -996,13 +1040,14 @@ class YEDB():
         Returns:
             Generator object with broken keys found and removed
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         if debug:
             logger.debug(
                 f'purge operation requested, keep_broken: {keep_broken}')
         dts = set()
         with self.lock:
+            self.cache.clear()
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             # clean up files
             for d in self.db.glob('**/*'):
                 if not d.is_dir() and d != self.lock_file and \
@@ -1041,12 +1086,12 @@ class YEDB():
         Returns:
             Generator object with broken keys found
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         if debug:
             logger.debug(f'check operation requested')
         broken = []
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             for d in self.db.glob('**/*'):
                 if d.suffix == self.suffix or d.suffix == '.tmp':
                     try:
@@ -1058,7 +1103,7 @@ class YEDB():
                             logger.debug(f'broken key file found: {d}')
                         yield str(d)[self._dbpath_len:-self._suffix_len]
 
-    def list_subkeys(self, key=''):
+    def list_subkeys(self, key='', hidden=False):
         """
         List subkeys of the specified key
 
@@ -1070,29 +1115,30 @@ class YEDB():
             values are yielded. To unlock the db earlier, convert the returned
             generator into a list
         """
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         name = self._fmt_key(key)
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             for f in self.db.glob(f'{name}/**/*{self.suffix}'
                                   if name else f'**/*{self.suffix}'):
                 name = f.absolute().as_posix()[self.
                                                _dbpath_len:-self._suffix_len]
-                yield name
+                if hidden or not name.startswith('.'):
+                    yield name
 
     def _delete_subkeys(self, name='', flush=False):
-        if not self._opened:
-            raise RuntimeError('database is not opened')
         name = self._fmt_key(name)
         import shutil
         dts = set()
         with self.lock:
+            if not self._opened:
+                raise RuntimeError('database is not opened')
             if name:
                 path = self.db / name
             else:
                 path = self.db
             try:
-                for k in reversed(sorted(self.list_subkeys(name))):
+                for k in reversed(sorted(self.list_subkeys(name, hidden=True))):
                     self.delete(k, _no_flush=True)
                     try:
                         del self._key_locks[k]
@@ -1116,8 +1162,9 @@ class YEDB():
                     self._sync_dirs(dts)
             except FileNotFoundError:
                 pass
+            self._purge_cache_by_path(name)
 
-    def get_subkeys(self, key='', ignore_broken=False):
+    def get_subkeys(self, key='', ignore_broken=False, hidden=False):
         """
         Get subkeys of the specified key and their values
 
@@ -1135,7 +1182,7 @@ class YEDB():
         if not self._opened:
             raise RuntimeError('database is not opened')
         name = self._fmt_key(key)
-        for key in self.list_subkeys(name):
+        for key in self.list_subkeys(name, hidden=hidden):
             try:
                 yield key, self.get(key)
             except:
@@ -1243,7 +1290,10 @@ class KeyDict:
                     self.key_name, {}) != self.data:
                 if debug:
                     logger.debug(f'requesting to update {self.key_name}')
-                self.db._write(self.key_file, self.db._dump_value(self.data))
+                with self.db.lock:
+                    val, checksum, stime = self.db._dump_value(self.data)
+                    self.db._write(self.key_file, val)
+                    self.db.cache[self.key_name] = (self.data, checksum, stime)
         self._lock.release()
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -1320,7 +1370,10 @@ class KeyList:
                     self.key_name, {}) != self.data:
                 if debug:
                     logger.debug(f'requesting to update {self.key_name}')
-                self.db._write(self.key_file, self.db._dump_value(self.data))
+                with self.db.lock:
+                    val, checksum, stime = self.db._dump_value(self.data)
+                    self.db._write(self.key_file, val)
+                    self.db.cache[self.key_name] = (self.data, checksum, stime)
         self._lock.release()
 
     def __exit__(self, exc_type, exc_val, exc_tb):
