@@ -184,13 +184,13 @@ def cli():
     class KeyCompleter:
 
         def __call__(self, prefix, **kwargs):
-            for k in db.list_subkeys():
+            for k in db.list_subkeys(hidden=True):
                 yield k
 
     class KeyGroupCompleter:
 
         def __call__(self, prefix, **kwargs):
-            for k in db.list_subkeys():
+            for k in db.list_subkeys(hidden=True):
                 if '/' in k:
                     c = k.split('/')
                     for i in range(len(c) + 1):
@@ -229,7 +229,8 @@ def cli():
                 for k, v in d.items():
                     if v is None:
                         d[k] = '<null>'
-            from rapidtables import format_table, FORMAT_GENERATOR, MULTILINE_ALLOW
+            from rapidtables import (format_table, FORMAT_GENERATOR,
+                                     MULTILINE_ALLOW)
 
             header, rows = format_table(data,
                                         fmt=FORMAT_GENERATOR,
@@ -246,7 +247,9 @@ def cli():
                 if kwargs.get('recursive'):
                     data = []
                     for k, v in db.get_subkeys(
-                            key=key, ignore_broken=kwargs.get('ignore_broken')):
+                            key=key,
+                            ignore_broken=kwargs.get('ignore_broken'),
+                            hidden=kwargs.get('all')):
                         tp = type_name(v)
                         if isinstance(v, dict) or isinstance(v, list):
                             import json
@@ -350,9 +353,8 @@ def cli():
                 except KeyError:
                     value = ''
                 editor = os.getenv('EDITOR', 'vi')
-                tmpfile = Path(
-                    f'{tempfile.gettempdir()}/{random.randint(0,100000)}.tmp.yaml'
-                )
+                tmpfile = Path(f'{tempfile.gettempdir()}'
+                               f'/{random.randint(0,100000)}.tmp.yaml')
                 tmpfile.write_text('' if value == '' else yaml.
                                    dump(value, default_flow_style=False))
                 try:
@@ -407,7 +409,7 @@ def cli():
             elif cmd == 'ls':
                 key = kwargs.get('KEY')
                 data = []
-                for k in db.list_subkeys(key=key):
+                for k in db.list_subkeys(key=key, hidden=kwargs.get('all')):
                     data.append(dict(key=k))
                 if db.key_exists(key=key):
                     data.append(dict(key=key))
@@ -512,19 +514,24 @@ def cli():
                                           (time.perf_counter() - start)),
                                     color='yellow')))
                 print()
-                for n, v in [('numeric', 777.777), ('string', 'x' * 1000),
-                             ('array', test_arr), ('dict', test_dict)]:
-                    start = time.perf_counter()
-                    for z in range(iters):
-                        db.get(key=f'.benchmark/{n}/key{z}')
-                    print(
-                        colored(
-                            f'get/{n}'.ljust(12), color='green', attrs='bold') +
-                        ': {} keys/sec'.format(
-                            colored(round(iters /
-                                          (time.perf_counter() - start)),
-                                    color='yellow')))
-                print()
+                list(db.purge())
+                for c in range(2):
+                    for n, v in [('numeric', 777.777), ('string', 'x' * 1000),
+                                 ('array', test_arr), ('dict', test_dict)]:
+                        start = time.perf_counter()
+                        for z in range(iters):
+                            db.get(key=f'.benchmark/{n}/key{z}')
+                        print(
+                            colored(f'get{"(cached)" if c else ""}/{n}'.ljust(
+                                12),
+                                    color='green',
+                                    attrs='bold') +
+                            ': {} keys/sec'.format(
+                                colored(round(iters /
+                                              (time.perf_counter() - start)),
+                                        color='yellow')))
+                    print()
+                print('Cached entries: ', db.info()['cached'])
                 print('cleaning up...')
                 db.delete(key='.benchmark', recursive=True)
             elif cmd == 'clear':
@@ -550,6 +557,10 @@ def cli():
     ap_get.add_argument('KEY', help='Key name or <key>:<field> for dict keys'
                        ).completer = KeyGroupCompleter()
     ap_get.add_argument('-r', '--recursive', action='store_true')
+    ap_get.add_argument('-a',
+                        '--all',
+                        action='store_true',
+                        help='Include hidden')
     ap_get.add_argument('-y',
                         '--full',
                         action='store_true',
@@ -599,6 +610,10 @@ def cli():
     ap_ls = sp.add_parser('ls', help='List keys')
     ap_ls.add_argument('KEY', help='Root key, optional',
                        nargs='?').completer = KeyCompleter()
+    ap_ls.add_argument('-a',
+                       '--all',
+                       action='store_true',
+                       help='Include hidden')
 
     ap_info = sp.add_parser('info', help='Database info')
     ap_info.add_argument('-y', '--full', action='store_true')
